;; (let ((txt (with-current-buffer "ansi-test.txt" (buffer-string))))
;;   (list (time (dotimes (_ 20) (ansi-color-apply txt)))
;; 	(time (dotimes (_ 20) (ansi-color-apply2 txt)))))

(require 'ansi-color)

(defmacro time (&rest forms)
  `(let ((t1 (float-time)))
     (progn ,@forms)
     (- (float-time) t1)))

;;; original version
(defun ansi-color-apply (string)
  (let ((face (car ansi-color-context))
	(start 0) end escape-sequence result
	colorized-substring)
    ;; If context was saved and is a string, prepend it.
    (if (cadr ansi-color-context)
        (setq string (concat (cadr ansi-color-context) string)
              ansi-color-context nil))
    ;; Find the next escape sequence.
    (while (setq end (string-match ansi-color-regexp string start))
      (setq escape-sequence (match-string 1 string))
      ;; Colorize the old block from start to end using old face.
      (when face
	(put-text-property start end 'ansi-color t string)
	(put-text-property start end 'face face string))
      (setq colorized-substring (substring string start end)
	    start (match-end 0))
      ;; Eliminate unrecognized ANSI sequences.
      (while (string-match ansi-color-drop-regexp colorized-substring)
	(setq colorized-substring
	      (replace-match "" nil nil colorized-substring)))
      (push colorized-substring result)
      ;; Create new face, by applying escape sequence parameters.
      (setq face (ansi-color-apply-sequence escape-sequence face)))
    ;; if the rest of the string should have a face, put it there
    (when face
      (put-text-property start (length string) 'ansi-color t string)
      (put-text-property start (length string) 'face face string))
    ;; save context, add the remainder of the string to the result
    (let (fragment)
      (if (string-match "\033" string start)
	  (let ((pos (match-beginning 0)))
	    (setq fragment (substring string pos))
	    (push (substring string start pos) result))
	(push (substring string start) result))
      (if (or face fragment)
	  (setq ansi-color-context (list face fragment))
	(setq ansi-color-context nil)))
    (apply 'concat (nreverse result))))

;;; my version
(defun ansi-color-apply2 (string)
  (let ((face (car ansi-color-context))
        start end fragment escape-sequence)
    ;; If context was saved and is a string, prepend it.
    (if (cadr ansi-color-context)
        (setq string (concat (cadr ansi-color-context) string)
              ansi-color-context nil))
    (prog1
        (with-temp-buffer
          (insert string)
          (setq start (point-min-marker))
          (goto-char start)
          (while (re-search-forward ansi-color-drop-regexp nil t)
            (replace-match ""))
          (goto-char start)
          ;; Find the next escape sequence.
          (while (re-search-forward ansi-color-regexp nil t)
            (setq end (match-beginning 0))
            (when face
              (put-text-property start end 'ansi-color t)
              (put-text-property start end 'face face))
            (setq start (copy-marker (match-end 0)))
            (setq escape-sequence (match-string 1))
            (replace-match "")
            (setq face (ansi-color-apply-sequence escape-sequence face)))
          ;; Search for the possible start of a new escape sequence
          (if (re-search-forward "\033" nil t)
              (setq fragment
                    (delete-and-extract-region (match-beginning 0)
                                               (point-max))))
          ;; If the rest of the string should have a face, put it there
          (when face
            (put-text-property start (point-max) 'ansi-color t)
            (put-text-property start (point-max) 'face face))
          ;; Return the string
          (buffer-string))
      ;; Save context; NB: ansi-color-context is buffer-local so set it after
      ;; exit the temp buffer.
      (if (or face fragment)
          (setq ansi-color-context (list face fragment))
        (setq ansi-color-context nil)))))
